#define _USE_MATH_DEFINES

#include "camera.h"
#include <math.h>

namespace OpenglTest
{

	camera::camera()
	{
		offest = 0.1f;
		yangle = 0.0f;
		zangle = M_PI;
		direct.z = -1.0f;
	}

	void camera::setThree(bool three)
	{
		bthree = three;
		zangle += M_PI;
		yangle *= -1;
	}

	vector3 camera::getEye() const
	{
		if (bthree)
		{
			return this->direct;
		}
		else
		{
			return this->origin;
		}
	}

	vector3 camera::getTarget() const
	{
		if (bthree)
		{
			return this->origin;
		}
		else
		{
			return this->direct;
		}
	}

	void camera::updateDirection()
	{
		if (zangle > M_PI * 2.0f)
		{
			zangle -= M_PI * 2.0f;
		}
		if (zangle < 0)
		{
			zangle += M_PI * 2.0f;
		}

		float len = 1.0f;
		if (bthree)
		{
			len = lenght == 0 ? 1.0f : lenght;
		}
		auto xy = cos(yangle) * len;
		direct.x = origin.x + xy*sin(zangle);
		direct.y = origin.y + len*sin(yangle);
		direct.z = origin.z + xy*cos(zangle);
	}

	void camera::move(float x, float y, float z)
	{
		auto sinz = sin(zangle);
		auto cosz = cos(zangle);
		auto xstep = x*cosz + z*sinz;
		auto zstep = z*cosz - x*sinz;
		if (bthree)
		{
			xstep *= -1;
			zstep *= -1;
		}
		origin.x += xstep;
		origin.y += y;
		origin.z += zstep;
		updateDirection();
	}

	void camera::rotate(float zang, float yang)
	{
		zangle -= zang;
		yangle += bthree ? -yang : yang;
		updateDirection();
	}

	void camera::lookat()
	{
		updateDirection();
		glMatrixMode(GL_MODELVIEW);							// ѡģ͹۲
		glLoadIdentity();
		auto eye = getEye();
		auto center = getTarget();
		gluLookAt(eye.x, eye.y, eye.z, center.x, center.y, center.z, 0.f, 1.f, 0.f);
	}

	void camera::mousefunc(int mouse, int state, int x, int y)
	{
		if (!firstMove)
		{
			auto ox = (mouseX - x)*offest*0.1f;
			auto oy = (mouseY - y)*offest*-0.1;
			rotate(ox, oy);
		}
		mouseX = x;
		mouseY = y;
	}

	void camera::keyboardfunc(unsigned char key, int x, int y)
	{
		switch (key)
		{
		case 'e':
		case 'E':
			this->move(0.f, 0.f, offest);
			break;
		case 'd':
		case 'D':
			this->move(0.f, 0.f, -offest);
			break;
		case 's':
		case 'S':
			this->move(offest, 0.f, 0.f);
			break;
		case 'f':
		case 'F':
			this->move(-offest, 0.f, 0.f);
			break;
		case 'w':
		case 'W':
			this->move(0.f, offest, 0.f);
			break;
		case 'r':
		case 'R':
			this->move(0.f, -offest, 0.f);
			break;
		case 'v':
		case 'V':
			bthree = !bthree;
			break;
		case '<':
			offest += 0.1f;
			break;
		case '>':
			offest -= 0.1f;
			break;
		default:
			break;
		}
	}
	
	void camera::setZangle(float angle)
	{
		zangle = angle;
		updateDirection();
	}

	void camera::setYangle(float angle)
	{
		yangle = angle;
		updateDirection();
	}
	camera::~camera()
	{
	}
}
