#include "plane.h"

namespace OpenglTest
{
	plane::plane(int xres, int yres, float xscale, float yscale, float height)
	{
		xr = xres;
		yr = yres;
		xc = xscale;
		yc = yscale;
		xl = (xr - 1)*xc;
		yl = (yr - 1)*yc;
		this->height = height;
	}

	void plane::init()
	{
		vector<float> vectors;
		vector<int> indexs;

		auto helfx = (xr - 1)*xc*0.5f;
		auto helfy = (yr - 1)*yc*0.5f;
		for (int y = 0; y < yr; y++)
		{
			for (int x = 0; x < xr; x++)
			{
				//uv
				vectors.push_back((float)x / (xr - 1));
				vectors.push_back((float)y / (yr - 1));
				//position
				vectors.push_back(xc*x - helfx);
				vectors.push_back(height);
				vectors.push_back(yc*y - helfy);
			}
		}
		for (int y = 0; y < yr - 1; y++)
		{
			for (int x = 0; x < xr - 1; x++)
			{
				indexs.push_back(y*xr + x);
				indexs.push_back((y + 1)*xr + x);
				indexs.push_back(y*xr + x + 1);
				indexs.push_back(y*xr + x + 1);
				indexs.push_back((y + 1)*xr + x);
				indexs.push_back((y + 1)*xr + x + 1);
			}
		}

		glGenBuffers(1, &vboId);
		glBindBuffer(GL_ARRAY_BUFFER, vboId);
		glBufferData(GL_ARRAY_BUFFER, vectors.size() * 4, vectors.data(), GL_STATIC_DRAW);

		glGenBuffers(1, &eboId);
		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, eboId);
		glBufferData(GL_ELEMENT_ARRAY_BUFFER, indexs.size() * 4, indexs.data(), GL_STATIC_DRAW);

		indexCount = indexs.size();
		bCreate = true;
	}

	void plane::draw()
	{
		if (!this->bCreate)
		{
			this->init();
		}
		mat->draw();

		glPushMatrix();
		glTranslatef(position.x, position.y, position.z);

		glBindBuffer(GL_ARRAY_BUFFER, vboId);
		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, eboId);

		glInterleavedArrays(GL_T2F_V3F, 0, 0);
		//glInterleavedArrays(GL_V3F, 0, 0);
		glDrawElements(GL_TRIANGLES, indexCount, GL_UNSIGNED_INT, 0);
		glPopMatrix();
	}

	void plane::drawShader(int pos, int tex)
	{
		if (!this->bCreate)
		{
			this->init();
		}
		mat->draw();

		glBindBuffer(GL_ARRAY_BUFFER, vboId);
		glEnableVertexAttribArray(pos);
		glEnableVertexAttribArray(tex);
		glVertexAttribPointer(tex, 2, GL_FLOAT, false, 20, (void*)0);
		glVertexAttribPointer(pos, 3, GL_FLOAT, false, 20, (void*)8);
		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, eboId);
		glDrawElements(GL_TRIANGLES, indexCount, GL_UNSIGNED_INT, 0);
		glDisableVertexAttribArray(pos);
		glDisableVertexAttribArray(tex);
		
	}

	plane::~plane()
	{

	}
}