#pragma once
#include <math.h>
namespace OpenglTest
{
	class vector3
	{
	public:
		float x, y, z;
		static const vector3 Zero;
	public:
		inline vector3(){}
		inline vector3(float x, float y, float z)
			: x(x), y(y), z(x)
		{
		}

		inline vector3 operator +(const vector3& vector) const
		{
			return vector3(x + vector.x, y + vector.y, z + vector.z);
		}

		inline vector3 operator -(const vector3& vector) const
		{
			return vector3(x - vector.x, y - vector.y, z - vector.z);
		}

		inline vector3 operator *(const float scale) const
		{
			return vector3(x *scale, y *scale, z *scale);
		}

		inline vector3 operator /(const float scale) const
		{
			return vector3(x / scale, y / scale, z / scale);
		}

		inline float* ptr()
		{
			return &x;
		}

		inline float lenght() const
		{
			return sqrt(x*x + y*y + z*z);
		}

		inline float squaredLenght() const
		{
			return x*x + y*y + z*z;
		}

		inline float distance(const vector3 rhs) const
		{
			return (*this - rhs).lenght();
		}

		inline float squaredDistance(const vector3 rhs) const
		{
			return (*this - rhs).squaredLenght();
		}

		inline float dot(const vector3 rhs) const
		{
			return x*rhs.x + y*rhs.y + z*rhs.z;
		}

		inline vector3 cross(const vector3 rhs) const
		{
			return vector3(y*rhs.z - z*rhs.y, z*rhs.x - x*rhs.z, x*rhs.y - y* rhs.x);
		}
	};
}


